/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * Copyright (c) 2013 Oracle and/or its affiliates. All rights reserved.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common Development
 * and Distribution License("CDDL") (collectively, the "License").  You
 * may not use this file except in compliance with the License.  You can
 * obtain a copy of the License at
 * https://glassfish.dev.java.net/public/CDDL+GPL_1_1.html
 * or packager/legal/LICENSE.txt.  See the License for the specific
 * language governing permissions and limitations under the License.
 *
 * When distributing the software, include this License Header Notice in each
 * file and include the License file at packager/legal/LICENSE.txt.
 *
 * GPL Classpath Exception:
 * Oracle designates this particular file as subject to the "Classpath"
 * exception as provided by Oracle in the GPL Version 2 section of the License
 * file that accompanied this code.
 *
 * Modifications:
 * If applicable, add the following below the License Header, with the fields
 * enclosed by brackets [] replaced by your own identifying information:
 * "Portions Copyright [year] [name of copyright owner]"
 *
 * Contributor(s):
 * If you wish your version of this file to be governed by only the CDDL or
 * only the GPL Version 2, indicate your decision by adding "[Contributor]
 * elects to include this software in this distribution under the [CDDL or GPL
 * Version 2] license."  If you don't indicate a single choice of license, a
 * recipient has the option to distribute your version of this file under
 * either the CDDL, the GPL Version 2 or to extend the choice of license to
 * its licensees as provided above.  However, if you add GPL Version 2 code
 * and therefore, elected the GPL Version 2 license, then the option applies
 * only if the new code is made subject to such option by the copyright
 * holder.
 */
package org.jvnet.hk2.testing.junit;

import junit.framework.Assert;

import org.glassfish.hk2.api.DynamicConfiguration;
import org.glassfish.hk2.api.DynamicConfigurationService;
import org.glassfish.hk2.api.ServiceLocator;
import org.glassfish.hk2.api.ServiceLocatorFactory;

/**
 * These are other useful test utilities that can be used
 * @author jwells
 *
 */
public class HK2TestUtilities {
    private final static ServiceLocatorFactory factory = ServiceLocatorFactory.getInstance();
    
    public static ServiceLocator create(String name, HK2TestModule... modules) {
        return create(name, null, modules);
    }
    
    /**
     * Will create a ServiceLocator after doing test-specific bindings from the TestModule
     * 
     * @param name The name of the service locator to create.  Should be unique per test, otherwise
     * this method will fail.
     * @param parent The parent locator this one should have.  May be null
     * @param modules The test modules, that will do test specific bindings.  May be null
     * @return A service locator with all the test specific bindings bound
     */
    public static ServiceLocator create(String name, ServiceLocator parent, HK2TestModule... modules) {
        ServiceLocator retVal = factory.find(name);
        Assert.assertNull("There is already a service locator of this name, change names to ensure a clean test: " + name, retVal);
        
        retVal = factory.create(name, parent);
        
        if (modules == null || modules.length <= 0) return retVal;
        
        DynamicConfigurationService dcs = retVal.getService(DynamicConfigurationService.class);
        Assert.assertNotNull("Their is no DynamicConfigurationService.  Epic fail", dcs);
        
        DynamicConfiguration dc = dcs.createDynamicConfiguration();
        Assert.assertNotNull("DynamicConfiguration creation failure", dc);
        
        for (HK2TestModule module : modules) {
            module.configure(dc);
        }
        
        dc.commit();
        
        return retVal;
    }

}
